@@ -3,6 +3,12 @@ import json
 import time
 import yaml
 import base64
+import threading
+from tqdm import tqdm
+import os
+import shutil
+from concurrent.futures import ThreadPoolExecutor
+import subprocess
 
 # This class will No longer be supported in version 4+ of the API
 class simulation:
@@ -131,7 +137,7 @@ class simulation:
         response = requests.request("POST", url, headers=headers, data=payload)
 
         print(response.text)
-
+    #phase out in future versions
     def downloadData(self):
         url = "https://lexsetapi.azurewebsites.net/api/datasets/getdatasetarchives?dataset_id=" + str(self.dataID)
 
@@ -141,17 +147,21 @@ class simulation:
         }
 
         response = requests.request("GET", url, headers=headers, data=payload)
+        getCode = response.status_code
 
-        print(response.text)
-        parseResponse = json.loads(response.text)
-        print("Response:")
-        print(response.text)
-        self.datasetURL = parseResponse[0]["url"]
-        if len(parseResponse) > 0:
-            resp = requests.get(self.datasetURL, allow_redirects=True)
-            open('dataset.zip', 'wb').write(resp.content)
-        else:
-            return("Dataset zipping")
+        if getCode == 401:
+            return("Unauthorized")
+        if getCode == 200:
+            print(response.text)
+            parseResponse = json.loads(response.text)
+            print("Response:")
+            print(response.text)
+            self.datasetURL = parseResponse[0]["url"]
+            if len(parseResponse) > 0:
+                resp = requests.get(self.datasetURL, headers=headers, allow_redirects=True)
+                open('dataset.zip', 'wb').write(resp.content)
+            else:
+                return("Dataset zipping")
 
 def getDatasetID(id,userToken):
     url = "https://lexsetapi.lexset.ai/api/simulations/getsimulationstatus?id=" + str(id)
@@ -283,6 +293,7 @@ def start(simulationID,token):
         print("Error: " + str(getCode))
         return(getCode)
 
+#phase out in future versions
 def download(datasetID ,userToken, localPath = "NONE"):
     url = "https://lexsetapi.azurewebsites.net/api/datasets/getdatasetarchives?dataset_id=" + str(datasetID)
 
@@ -302,10 +313,17 @@ def download(datasetID ,userToken, localPath = "NONE"):
     
     if getCode == 200:
         datasetURL = parseResponse[0]["url"]
-        resp = requests.get(datasetURL, allow_redirects=True)
+        resp = requests.get(datasetURL, headers=headers, allow_redirects=True)
         if(localPath == "NONE"):
             localPath = "dataset.zip"
-        open(localPath, 'wb').write(resp.content)
+        total_size = int(resp.headers.get('content-length', 0))
+        block_size = 1024 #1 Kibibyte
+        t=tqdm(total=total_size, unit='iB', unit_scale=True, desc=localPath.split("/")[-1])
+        with open(localPath, 'wb') as f:
+            for data in resp.iter_content(block_size):
+                t.update(len(data))
+                f.write(data)
+        t.close()
         output ={
             "local path": localPath,
             "dataset url": datasetURL,
@@ -398,15 +416,9 @@ class activateSimulation:
         }
 
         response = requests.request("POST", url, headers=headers, data=payload)
-        getCode = response.status_code
-        if getCode == 401:
-            return("Unauthorized")
-        if getCode == 200:
-            print('NEW JOB STARTED: ' + str(self.simID) + ' has started')
-            print('-------------')
-        else:
-            print("Error: " + str(getCode))
-            return(getCode)
+        # print('NEW JOB STARTED: ' + str(self.simID) + ' has started')
+        # print('-------------')
+
 
     def updateStatus(self):
         url = "https://lexsetapi.azurewebsites.net/api/simulations/getsimulationstatus?id=" + str(self.simID)
@@ -429,7 +441,7 @@ class activateSimulation:
             self.nodeCount = parseResponse["requestedNodeCount"]
             self.hasStarted = parseResponse["hasStarted"]
             self.isComplete = parseResponse["isComplete"]
-            
+
     def getProgress(self):
         url = "https://lexsetapi.azurewebsites.net/api/simulations/getstatus?id=" + str(self.simID)
 
@@ -439,16 +451,12 @@ class activateSimulation:
         }
 
         response = requests.request("GET", url, headers=headers, data=payload)
-        getCode = response.status_code
-        if getCode == 401:
-            return("Unauthorized")
-        if getCode == 200:
-            #update if sim is complete or not complete
-            parseResponse = json.loads(response.text)
-            print(parseResponse)
-        else:
-            print("Error: " + str(getCode))
-            return(getCode)
+
+        #update if sim is complete or not complete
+        parseResponse = json.loads(response.text)
+        # print(parseResponse)
+        return parseResponse['percentComplete']
+        #self.isComplete = parseResponse["isComplete"]
 
 
     def stopSimulation(self):
@@ -490,7 +498,7 @@ class activateSimulation:
             print(response.text)
             self.datasetURL = parseResponse[0]["url"]
             if len(parseResponse) > 0:
-                resp = requests.get(self.datasetURL, allow_redirects=True)
+                resp = requests.get(self.datasetURL, headers=headers, allow_redirects=True)
                 open('dataset.zip', 'wb').write(resp.content)
             else:
                 return("Dataset zipping")
@@ -508,17 +516,24 @@ def activeSimulationNodes(userID, token):
     }
 
     response = requests.request("GET", url, headers=headers, data=payload)
+    getCode = response.status_code
 
-    #update if sim is complete or not complete
-    parseResponse = json.loads(response.text)
-    nodes = 0
+    if getCode == 401:
+        return("Unauthorized")
+    if getCode == 200:
+        #update if sim is complete or not complete
+        parseResponse = json.loads(response.text)
+        nodes = 0
 
-    y = parseResponse
+        y = parseResponse
 
-    for item in range(len(y)):
-    
-        if (y[item]['hasStarted']== True) :
-            nodes = nodes + int(y[item]['requestedNodeCount'])
+        for item in range(len(y)):
+        
+            if (y[item]['hasStarted']== True) :
+                nodes = nodes + int(y[item]['requestedNodeCount'])
+    else:
+        print("Error: " + str(getCode))
+        return(getCode)
 
     #print(parseResponse)
     return nodes
@@ -617,3 +632,112 @@ def getOrganizationSimulations(orgID,state,token):
     else:
         print("Error: " + str(getCode))
         return(getCode)
+
+def getComputeResources(id,userToken): 
+    url = "https://seahaven.lexset.ai/api/accountusage/GetActiveComputeNodeCount?userid=" + str(id) 
+ 
+    payload={} 
+    headers = { 
+    'Authorization': 'Bearer ' + userToken 
+    } 
+ 
+    response = requests.request("GET", url, headers=headers, data=payload) 
+    getCode = response.status_code 
+    if getCode == 401: 
+        return("Unauthorized") 
+    if getCode == 200: 
+        parseResponse = json.loads(response.text) 
+        return(parseResponse) 
+    else: 
+        print("Error: " + str(getCode)) 
+        return(getCode) 
+
+# testing not for use
+def s3_transfer(id, userToken, bucketName, simulationId, token):
+
+    url = "https://coreapi.lexset.ai/api/userdatamanagement/TransferDatasetToS3?userid=" + str(id) + "&simulationId=" + str(simulationId) + "&bucketName=" + bucketName
+    
+    payload={}
+    headers = {
+    'Authorization': 'Bearer ' + token
+    }
+    response = requests.request("GET", url, headers=headers, data=payload)
+    getCode = response.status_code
+    if getCode == 401:
+        return("Unauthorized")
+    if getCode == 200:
+        parseResponse = json.loads(response.text)
+        return(parseResponse)
+    else:
+        print("Error: " + str(getCode))
+        return(getCode)
+
+
+def download_multiThread(id,userToken, localPath = "NONE",workers = 4):
+    url = "https://coreapi.lexset.ai/api/dataset/download/" + str(id) + "/dataset.zip"
+    bearer_token = userToken
+    
+    headers = {"Authorization": f"Bearer {bearer_token}"}
+
+    session = requests.Session()
+    response = session.get(url, headers=headers, stream=True)
+    if response.status_code == 401:
+        return("Unauthorized")
+    if response.status_code == 200:
+        file_size = int(response.headers["content-length"])
+    else:
+        print("Error: " + str(response.status_code))
+        return(response.status_code)
+    session.close()
+
+    def divide_into_parts(file_size, num_workers):
+        part_size = file_size // num_workers
+        part_starts = [i * part_size for i in range(num_workers)]
+        part_ends = [(i + 1) * part_size - 1 for i in range(num_workers - 1)] + [file_size - 1]
+        return list(zip(part_starts, part_ends))
+
+    def combine_bin_files(bin_files, destination):
+        with open(destination, "wb") as outfile:
+            for bin_file in bin_files:
+                with open(bin_file, "rb") as readfile:
+                    shutil.copyfileobj(readfile, outfile)
+
+    def delete_bin_files(parts):
+        for i in range(parts):
+            os.remove(f"part{i}.bin")
+
+    parts = divide_into_parts(file_size, workers)
+
+    #function that downloads a part of the file and saves it to disk
+    def download_part(start, end, part_number, progress_bar):
+        headers = {"Authorization": f"Bearer {bearer_token}", "Range": f"bytes={start}-{end}"}
+        response = requests.get(url, headers=headers, stream=True)
+
+        with open(f"part{part_number}.bin", "wb") as f:
+            for chunk in response.iter_content(chunk_size=1024*1024):
+                f.write(chunk)
+                progress_bar.update(len(chunk))
+
+    # create a progress bar for each part
+    progress_bars = [tqdm(total=end-start, desc=f"part_{i}", leave= False) for i, (start, end) in enumerate(parts)]
+
+    with ThreadPoolExecutor() as executor:
+        for i, (start, end) in enumerate(parts):
+            future = executor.submit(download_part, start, end, i, progress_bars[i])
+
+    if localPath == "NONE":
+        destination = "dataset.zip"
+    else:
+        destination = localPath
+
+    combine_bin_files([f"part{i}.bin" for i in range(len(parts))], destination)
+
+    delete_bin_files(workers)
+
+def delete_simulation(id, token):
+    headers = {'Authorization': 'Bearer ' + str(token)}
+    response = requests.delete(f'https://lexsetapi.lexset.ai/api/simulations/archivesimulation?id={id}', headers=headers)
+    if response.status_code == 200:
+        print("Simulation successfully deleted.")
+    else:
+        print(f"Error deleting simulation: {response.text}")
